
/*
 * Interface with PROATM WDM driver
 * This driver should work with all Windows OS
 */

#include <winsock2.h>
//#include <ansi_c.h>  // for LabWindows environment
#include <initguid.h>
#include <setupapi.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include "atm.h"
#include "proatmapi.h"
#include "wdmatmif.h"
#include "public.h"


static int GetDeviceDescription( IN  char *DevPath,
						         OUT char *OutBuffer,
						         IN ULONG OutBufferLen )
{
    HDEVINFO                            hardwareDeviceInfo = NULL;
    SP_DEVICE_INTERFACE_DATA            deviceInterfaceData;
    SP_DEVINFO_DATA                     deviceInfoData;
    DWORD                               dwRegType, error;
    int                                ret = FALSE;

    hardwareDeviceInfo = SetupDiCreateDeviceInfoList(NULL, NULL);
    if(INVALID_HANDLE_VALUE == hardwareDeviceInfo)
        goto Error;

    //
    // Enumerate devices
    //
    deviceInterfaceData.cbSize = sizeof(deviceInterfaceData);

    SetupDiOpenDeviceInterface (hardwareDeviceInfo, DevPath,
                                 0, &deviceInterfaceData);

    deviceInfoData.cbSize = sizeof(deviceInfoData);
    if(!SetupDiGetDeviceInterfaceDetail (
            hardwareDeviceInfo,
            &deviceInterfaceData,
            NULL, // probing so no output buffer yet
            0, // probing so output buffer length of zero
            NULL,
            &deviceInfoData) && (error = GetLastError()) != ERROR_INSUFFICIENT_BUFFER)
    {
        goto Error;
    }
    //
    // Get the friendly name for this instance. If that fails
    // try to get the device description.
    //

    if(!SetupDiGetDeviceRegistryProperty(hardwareDeviceInfo, &deviceInfoData,
                                     SPDRP_FRIENDLYNAME,
                                     &dwRegType,
                                     (char*) OutBuffer,
                                     OutBufferLen,
                                     NULL))
    {
        if(!SetupDiGetDeviceRegistryProperty(hardwareDeviceInfo, &deviceInfoData,
                                     SPDRP_DEVICEDESC,
                                     &dwRegType,
                                     (char*) OutBuffer,
                                     OutBufferLen,
                                     NULL))
		{
            goto Error;
        }
    }

    ret = TRUE;

Error:
   if(hardwareDeviceInfo)
        SetupDiDestroyDeviceInfoList (hardwareDeviceInfo);

	return ret;
}


BOOL WDMGetATMAdapters(PATM_ADAPTER pAdapters, DWORD *pAdapterCount)
{
    HDEVINFO                            hardwareDeviceInfo;
    SP_DEVICE_INTERFACE_DATA            deviceInterfaceData;
    PSP_DEVICE_INTERFACE_DETAIL_DATA    deviceInterfaceDetailData = NULL;
    ULONG                               predictedLength = 0;
    ULONG                               requiredLength = 0, i, j;
	GUID                                InterfaceGuid = GUID_DEVINTERFACE_PROATM;

    int DeviceNumber = 0;

    hardwareDeviceInfo = SetupDiGetClassDevs (
                       (LPGUID)&InterfaceGuid,
                       NULL, // Define no enumerator (global)
                       NULL, 
                       (DIGCF_PRESENT | // Only Devices present
                       DIGCF_DEVICEINTERFACE)); // Function class devices.
    
    if(INVALID_HANDLE_VALUE == hardwareDeviceInfo)
       goto Error;

    //
    // Enumerate devices of a specific interface class
    //
    deviceInterfaceData.cbSize = sizeof(deviceInterfaceData);

	j = *pAdapterCount;
    for(i=0; SetupDiEnumDeviceInterfaces (hardwareDeviceInfo,
                                 0, // No care about specific PDOs
                                 (LPGUID)&InterfaceGuid,
                                 i, //
                                 &deviceInterfaceData) && j <MAX_ATM_ADAPTERS; i++ ) 
	{

        //
        // Allocate a function class device data structure to
        // receive the information about this particular device.
        //

        //
        // First find out the required length of the buffer
        //
        if (deviceInterfaceDetailData) 
            HeapFree (GetProcessHeap(), 0, deviceInterfaceDetailData);

        if(!SetupDiGetDeviceInterfaceDetail (
                hardwareDeviceInfo,
                &deviceInterfaceData,
                NULL, // probing so no output buffer yet
                0, // probing so output buffer length of zero
                &requiredLength,
                NULL) && (GetLastError()) != ERROR_INSUFFICIENT_BUFFER)
        {
           goto Error;
        }
        predictedLength = requiredLength;

        deviceInterfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)HeapAlloc(GetProcessHeap(), 
												HEAP_ZERO_MEMORY, predictedLength);
        if (deviceInterfaceDetailData == NULL) 
            goto Error;

        deviceInterfaceDetailData->cbSize =  sizeof (SP_DEVICE_INTERFACE_DETAIL_DATA);


        if (! SetupDiGetDeviceInterfaceDetail (
                   hardwareDeviceInfo,
                   &deviceInterfaceData,
                   deviceInterfaceDetailData,
                   predictedLength,
                   &requiredLength,
                   NULL))
		{
            goto Error;
        }
        
	    if(!GetDeviceDescription( deviceInterfaceDetailData->DevicePath,
							      pAdapters[j].DriverDesc,
								  sizeof(pAdapters[j].DriverDesc)))
		{
			goto Error;
		}

		// Copy the device path so that we can open the device using CreateFile.
		strncpy( pAdapters[j].PnpInstanceID, deviceInterfaceDetailData->DevicePath, 
													sizeof(pAdapters[j].PnpInstanceID));
		pAdapters[j].AdapterType = WDM_ADAPTER; 
		pAdapters[j].DeviceNumber  = DeviceNumber++;
		pAdapters[j].AdapterNumber = j;
		if (++j > MAX_ATM_ADAPTERS)
			break;
    }

	*pAdapterCount =j;
    if(deviceInterfaceDetailData)
        HeapFree (GetProcessHeap(), 0, deviceInterfaceDetailData);

    SetupDiDestroyDeviceInfoList (hardwareDeviceInfo);
    return TRUE;

Error:

    if(deviceInterfaceDetailData)
        HeapFree (GetProcessHeap(), 0, deviceInterfaceDetailData);

    SetupDiDestroyDeviceInfoList (hardwareDeviceInfo);
    return FALSE;
}


HANDLE WDMOpenATMSocket( PATM_ADAPTER pAdapter)
{
    HANDLE                  hSocket;

    // Open an handle to the device.
    hSocket = CreateFile (
			pAdapter->PnpInstanceID,
            GENERIC_READ | GENERIC_WRITE,
            0,
            NULL, // no SECURITY_ATTRIBUTES structure
            OPEN_EXISTING, // No special create flags
            FILE_FLAG_OVERLAPPED,
            NULL);

 
    return hSocket;

}

int WDMCloseATMSocket( HANDLE hSocket)
{
 	if(hSocket != INVALID_HANDLE_VALUE)
	{
		CloseHandle( hSocket );
		return TRUE;
	}
	return FALSE;
}

BOOL WDMGetATMInformation ( PATM_ADAPTER pAdapter)
{
	LPPROTOCOL_DRIVER_REQUEST pPROATMRequest=NULL;
    ULONG RequestLength;
    int bResult = FALSE;
    PATM_APPLICATION_INFO pInfo;
	HANDLE handle;

	if ((handle =WDMOpenATMSocket(pAdapter)) ==  INVALID_HANDLE_VALUE) {
		printf ("Could not open the adapter: %d\n", GetLastError());
		return FALSE;
	}

    RequestLength = offsetof(PROTOCOL_OID_DATA, Data) + sizeof(ATM_APPLICATION_INFO);
    pPROATMRequest = PROATMAllocateRequestData(RequestLength);
    if ( pPROATMRequest )
    {
        #define pOidData ((PPROTOCOL_OID_DATA)(pPROATMRequest->Buffer))

	    pOidData->Oid    = OID_ATM_APPLICATION_INFO;
	    pOidData->Length = sizeof(ATM_APPLICATION_INFO );

        bResult = PROATMDeviceIoControl( handle,
								             pPROATMRequest,
								             IOCTL_ATM_QUERY_OID_VALUE,
								             TRUE );

        if ( bResult && (pPROATMRequest->ulBytesReceived >= RequestLength) )
        {
	        pInfo  = (PATM_APPLICATION_INFO)pOidData->Data;
            pAdapter->MemorySize = pInfo->MemorySize;
            pAdapter->Speed      = pInfo->Speed;
            pAdapter->LinkStatus = pInfo->LinkStatus;
            memcpy(&pAdapter->MacAddress, pInfo->MacAddress,6);
            pAdapter->DriverVersion = pInfo->DriverVersion;
     	    bResult = TRUE;

        }
 
    }
    PROATMFreeRequestData(pPROATMRequest);
 
	WDMCloseATMSocket(handle);

    return bResult;

}



/*++

Description: Thread that receives frames on a given VC
    
Arguments: param = pointer to the VC


--*/
static DWORD WDMReceiveThread(LPVOID param)
{
	DWORD  bytesRecv;
    int i,iError;
	DWORD dwEvent;

	PPDU        pRecvPDU[MAX_RECV_PDUS+1];  // only pRecvPDU[MAX_RECV_PDUS] will be used
    OVERLAPPED  oRecvOverlapped[MAX_RECV_PDUS +1];
    HANDLE	    hRecvEventList[MAX_RECV_PDUS +1];
	
	PATM_PVC pVC	= (PATM_PVC)param;
	HANDLE handle	=pVC->hSocket;
	UINT MaxPacket	= pVC->ATMSettings.PacketSize;
	PLIST_HEAD pReceiveQueue   = &pVC->ReceiveQueue;
	int	CurrentIndex=0;

	iError = 0;	
	
	for (i = 0; i<MAX_RECV_PDUS+1; i++)
	{
		pRecvPDU[i]=NULL;
		memset(&oRecvOverlapped[i],0, sizeof(OVERLAPPED));
		hRecvEventList[i] = NULL;
	}

	// Provide some buffers in advance for the Read process
	for (i=0; i < MAX_RECV_PDUS; i++)
	{
		oRecvOverlapped[i].hEvent= CreateEvent(0, FALSE, FALSE, NULL);
		if (oRecvOverlapped[i].hEvent == NULL)
		{ 
			iError = GetLastError();
            break;
		}
		if ((pRecvPDU[i] = (PPDU)calloc(1, sizeof(PDU) + MaxPacket )) == NULL)
		{
			iError = ERROR_NOT_ENOUGH_MEMORY;
			printf ("Not Enough Memory for Read Buffers\n");
            break;
		}
		pRecvPDU[i]->pBuffer = (char *)(pRecvPDU[i] +1);
		pRecvPDU[i]->uLength = MaxPacket;

		// The purpose of Read is only to feed the lower layers with free buffers
		if (!ReadFile( handle, pRecvPDU[i]->pBuffer, MaxPacket, &bytesRecv, &oRecvOverlapped[i] ))
		{
			iError = GetLastError();
			if (iError != ERROR_IO_PENDING)
			{
 				printf ("Read Failed: %d\n", iError);
 				break;
			}

			hRecvEventList[i]=oRecvOverlapped[i].hEvent;
		}
	}
 
	if (iError == 0 || iError == ERROR_IO_PENDING)
	{
		// Actual Read process 
		hRecvEventList[MAX_RECV_PDUS]=pVC->hReceiveCloseEvent;
		while(TRUE)
		{
			// Wait until one or all of the specified objects are in the signaled state
			dwEvent = WaitForMultipleObjects( MAX_RECV_PDUS+1, hRecvEventList, FALSE, INFINITE);

			if (dwEvent <  MAX_RECV_PDUS)
			{ 
				int bReceived = FALSE;
				CurrentIndex = dwEvent;
				iError = 0;
				
				// There is at least one event signaled about one of the provided buffers. 
				// Check all buffers. The index of the first signaled buffer is dwEvent.
				// Check events without waiting. If the operation is still pending, GetOverlappedResult () 
				// will return FALSE and GetLastError () will return ERROR_IO_INCOMPLETE.  
				
				while (GetOverlappedResult( handle, &oRecvOverlapped[CurrentIndex], &bytesRecv, FALSE) != 0)
				{
					PPDU pPDU = pRecvPDU[CurrentIndex];
					pRecvPDU[CurrentIndex] = NULL;
					pPDU->uLength = bytesRecv;
					MUTEX_LOCK(pVC->hReceiveMutex);
					
					// Chain the received buffer to the VC Receive Queue
					LIST_ADD_TAIL( &pPDU->Link, pReceiveQueue);
					MUTEX_UNLOCK(pVC->hReceiveMutex);

					// Allocate a new buffer to replace the used one
					if ((pRecvPDU[CurrentIndex] = (PPDU)calloc(1, sizeof(PDU) + MaxPacket )) == NULL)
					{
						iError = ERROR_NOT_ENOUGH_MEMORY;
						printf ("Not Enough Memory for Read Buffers\n");
						break;
					}
					// Set up the allocated buffer
					pRecvPDU[CurrentIndex]->pBuffer = (char *)(pRecvPDU[CurrentIndex] + 1);
					pRecvPDU[CurrentIndex]->uLength = MaxPacket;
					
					// Provide the new free buffer to the lower layers
					if (!ReadFile(handle, pRecvPDU[CurrentIndex]->pBuffer, MaxPacket, &bytesRecv, 
																		&oRecvOverlapped[CurrentIndex] ))
					{
						iError =GetLastError();
						if (iError != ERROR_IO_PENDING) 
						{
							printf ("Could not supply the new free buffer: %d\n", iError);
							break;
						}
					}
					else
						bReceived = TRUE;

					// Test Close event while receiving
					if (WaitForSingleObject(pVC->hReceiveCloseEvent,0) == WAIT_OBJECT_0)
					{
						bReceived = FALSE;
						iError = ERROR_OPERATION_ABORTED;
						// printf ("Read Operation ABorted\n");
						break;

					}
					CurrentIndex = (CurrentIndex +1)%MAX_RECV_PDUS;
				}
				
				if (!bReceived)
				{
					if (iError == 0)
						iError = GetLastError();
						
					if (iError != ERROR_IO_INCOMPLETE && iError != ERROR_IO_PENDING)
					{
						printf ("Read Failed: %d\n", iError);
						break;
					}
				}

			}
			else
			{
				// Error case. Close event. 
				iError = ERROR_OPERATION_ABORTED;
				// printf ("Read Operation Aborted\n");
				break;
			}

		}
	}
	//	Cancel all pending reads 
	CancelIo( handle);

	// Free resources 
	for (i = 0; i <MAX_RECV_PDUS+1; i++)
	{
		if (oRecvOverlapped[i].hEvent)
			CloseHandle(oRecvOverlapped[i].hEvent);
		
		if (pRecvPDU[i])
		{
			free(pRecvPDU[i]);
			pRecvPDU[i] = NULL;
		}
	}

	return iError;
}


BOOL WDMCreateATMPVC( PATM_PVC pVC)
{

	LPPROTOCOL_DRIVER_REQUEST   pPROATMRequest=NULL;
	PATM_SETTINGS   pSettings;
	ATM_PARAMETERS	AtmParameters;
	int bResult= FALSE;
	UINT  RequestLength;

	if ((pVC->hSocket=WDMOpenATMSocket(pVC->pAdapter)) == INVALID_HANDLE_VALUE)
		return FALSE;

	pSettings = &pVC->ATMSettings;
	memset( &AtmParameters, 0, sizeof(ATM_PARAMETERS)) ;

	if (pSettings->ReceiveFlow)
		AtmParameters.Flags	  |=  ENABLE_RECEIVE;
	if (pSettings->XmitFlow)
		AtmParameters.Flags	  |=  ENABLE_TRANSMIT;
	
	if (LAYER_AAL5 == pSettings->AalType )
		AtmParameters.AALType = AAL_TYPE_AAL5;
	else
		AtmParameters.AALType = AAL_TYPE_AAL0;

	AtmParameters.ConnectionId.Vpi = pSettings->VcSettings.Pvc.Vpi;
	AtmParameters.ConnectionId.Vci = pSettings->VcSettings.Pvc.Vci;

	if ( pSettings->QualityOfServices == QOS_UBR)
	{
		AtmParameters.Transmit.ServiceCategory = ATM_SERVICE_CATEGORY_UBR;
		AtmParameters.Transmit.PeakCellRate    = pSettings->QosSettings.Ubr.PeakCellRate;
		AtmParameters.Transmit.MaxSduSize	   = pSettings->PacketSize;
	}
	else if ( pSettings->QualityOfServices == QOS_CBR)
	{
		AtmParameters.Transmit.ServiceCategory = ATM_SERVICE_CATEGORY_CBR;
		AtmParameters.Transmit.PeakCellRate    = pSettings->QosSettings.Cbr.PeakCellRate;
		AtmParameters.Transmit.MaxSduSize	   = pSettings->PacketSize;

	}
	else if ( pSettings->QualityOfServices == QOS_VBR)
	{
		AtmParameters.Transmit.ServiceCategory  = ATM_SERVICE_CATEGORY_CBR;
		AtmParameters.Transmit.PeakCellRate     = pSettings->QosSettings.Vbr.PeakCellRate;
		AtmParameters.Transmit.AverageCellRate  = pSettings->QosSettings.Vbr.SubstainableCellRate;
		AtmParameters.Transmit.BurstLengthCells = pSettings->QosSettings.Vbr.MaxBurstSize;
	}

	RequestLength =	 sizeof(ATM_PARAMETERS);

	pPROATMRequest = PROATMAllocateRequestData(RequestLength);
	if ( pPROATMRequest)
	{
		memcpy( pPROATMRequest->Buffer, &AtmParameters, sizeof(ATM_PARAMETERS));
		bResult = PROATMDeviceIoControl(pVC->hSocket, pPROATMRequest,
											  IOCTL_ATM_CREATE_VC,
											  TRUE );
    }
    PROATMFreeRequestData(pPROATMRequest);
 
	if (!bResult)
	{
		WDMCloseATMSocket(pVC->hSocket);
		return FALSE;
	}

    pVC->hReceiveCloseEvent = CreateEvent(0, FALSE, FALSE, NULL);
    if (pVC->hReceiveCloseEvent == NULL)
    {
		WDMDeleteATMPVC(pVC);
        return FALSE;
    }
	
	// Create the transmit and receive mutexes the first time the VC is opened
	if (pVC->hReceiveMutex == NULL) 
		MUTEX_CREATE(pVC->hReceiveMutex); 

	if (pVC->hTransmitMutex == NULL) 
		MUTEX_CREATE(pVC->hTransmitMutex); 

	if ((pVC->hReceiveThreadHandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)WDMReceiveThread, (LPVOID)pVC, 0, NULL)) == 0)
    {
		WDMDeleteATMPVC(pVC);
        return FALSE;
    }

    return TRUE;

}


BOOL WDMDeleteATMPVC( PATM_PVC pVC)
{
	HANDLE handle;
	int bResult= FALSE;

	// Before closing the vc, stop the receiving thread
	if (pVC->hReceiveThreadHandle)
	{
		// Signal the event to close the receive thread.
		SetEvent(pVC->hReceiveCloseEvent);

		// Wait for it to exit. 
		WaitForSingleObject(pVC->hReceiveThreadHandle, INFINITE);
	}

	MUTEX_LOCK(pVC->hTransmitMutex);		// Do not write to vc when we are closing it
	handle =pVC->hSocket;

	if (handle != INVALID_HANDLE_VALUE)
	{

		LPPROTOCOL_DRIVER_REQUEST   pPROATMRequest;
		pVC->hSocket = INVALID_HANDLE_VALUE;

		pPROATMRequest = PROATMAllocateRequestData(0);
		if ( pPROATMRequest)
		{
			bResult = PROATMDeviceIoControl( handle,pPROATMRequest,
											  IOCTL_ATM_CLOSE_VC,
											  FALSE );
			PROATMFreeRequestData(pPROATMRequest);
		}
		WDMCloseATMSocket(handle);
	}
	MUTEX_UNLOCK(pVC->hTransmitMutex);		

	return bResult;
}

int WDMWriteATMPVC( PATM_PVC pVC, char *pBuffer, int bufSize)
{
	DWORD dwResult;
	OVERLAPPED oXmitOverlapped;
	int iReturn = -2;

	if(	pVC->bOpened && pVC->hSocket != INVALID_HANDLE_VALUE)
	{
		memset(&oXmitOverlapped, 0,sizeof(OVERLAPPED));
		oXmitOverlapped.hEvent= CreateEvent(0, FALSE, FALSE, NULL);

		if(!WriteFile( pVC->hSocket, pBuffer, bufSize,  &iReturn, &oXmitOverlapped))
		{
			dwResult = GetLastError();
			if (dwResult == ERROR_IO_PENDING)
			{
				dwResult= WaitForSingleObject(oXmitOverlapped.hEvent,1000);
				if (dwResult == WAIT_FAILED)
				{
					CloseHandle(oXmitOverlapped.hEvent);
					iReturn = -3;
				}
				else if (dwResult == WAIT_TIMEOUT || dwResult == WAIT_ABANDONED)
				{
					CloseHandle(oXmitOverlapped.hEvent);
					iReturn = -4;
				}
				else
				{
					dwResult=GetOverlappedResult(pVC->hSocket, &oXmitOverlapped, &iReturn, FALSE);
					if (dwResult == FALSE)
					{
						CloseHandle(oXmitOverlapped.hEvent);
						iReturn = -5;
					}
				}
			}
			else
			{
				CloseHandle(oXmitOverlapped.hEvent);
				iReturn = -6;
			}
		}
	}
	return iReturn;
}