/*++

Copyright (c) Microsoft Corporation. All rights reserved.
Copyright (c) PROSUM. All rights reserved.

Module Name:

	atm.h

Abstract:

	This module defines the structures, macros, and manifests available
	to ATM aware components. It is used by WDM Driver for ndis compatibility

Author:

	NDIS/ATM Development Team
	

Revision History:

	Initial Version - March 1996

--*/

#ifndef	_ATM_H_
#define	_ATM_H_

//
// Address type
//
typedef UINT32	ATM_ADDRESSTYPE;

//
// AAL types for winsock api
//
#define ATMPROTO_AALUSER           0x00   /* User-defined AAL */
#define ATMPROTO_AAL1              0x01   /* AAL 1 */
#define ATMPROTO_AAL2              0x02   /* AAL 2 */
#define ATMPROTO_AAL34             0x03   /* AAL 3/4 */
#define ATMPROTO_AAL5              0x05   /* AAL 5 */


//
// AAL types that the ATM driver supports
//
#define	AAL_TYPE_AAL0			1
#define	AAL_TYPE_AAL1			2
#define	AAL_TYPE_AAL34			4
#define	AAL_TYPE_AAL5			8

typedef UINT32	ATM_AAL_TYPE, *PATM_AAL_TYPE;


//
// AAL Parameters
//
typedef struct _AAL1_PARAMETERS
{
	unsigned char						Subtype;
	unsigned char						CBRRate;
	UINT16						Multiplier;
	unsigned char						SourceClockRecoveryMethod;
	unsigned char						ErrorCorrectionMethod;
	UINT16						StructuredDataTransferBlocksize;
	unsigned char						PartiallyFilledCellsMethod;
} AAL1_PARAMETERS, *PAAL1_PARAMETERS;

typedef struct _AAL34_PARAMETERS
{
	UINT16						ForwardMaxCPCSSDUSize;
	UINT16						BackwardMaxCPCSSDUSize;
	UINT16						LowestMID;
	UINT16						HighestMID;
	unsigned char						SSCSType;
} AAL34_PARAMETERS, *PAAL34_PARAMETERS;

typedef struct _AAL5_PARAMETERS
{
	UINT32						ForwardMaxCPCSSDUSize;
	UINT32						BackwardMaxCPCSSDUSize;
	unsigned char						Mode;
	unsigned char						SSCSType;
} AAL5_PARAMETERS, *PAAL5_PARAMETERS;

typedef struct _AALUSER_PARAMETERS
{
	UINT32						UserDefined;
} AALUSER_PARAMETERS, *PAALUSER_PARAMETERS;

typedef struct _AAL_PARAMETERS_IE
{
	ATM_AAL_TYPE				AALType;
	union
	{
		AAL1_PARAMETERS			AAL1Parameters;
		AAL34_PARAMETERS		AAL34Parameters;
		AAL5_PARAMETERS			AAL5Parameters;
		AALUSER_PARAMETERS		AALUserParameters;
	} AALSpecificParameters;

} AAL_PARAMETERS_IE, *PAAL_PARAMETERS_IE;

//
// ATM Traffic Descriptor
//
typedef struct _ATM_TRAFFIC_DESCRIPTOR	// For one direction
{
	UINT32						PeakCellRateCLP0;
	UINT32						PeakCellRateCLP01;
	UINT32						SustainableCellRateCLP0;
	UINT32						SustainableCellRateCLP01;
	UINT32						MaximumBurstSizeCLP0;
	UINT32						MaximumBurstSizeCLP01;
	BOOLEAN						Tagging;
} ATM_TRAFFIC_DESCRIPTOR, *PATM_TRAFFIC_DESCRIPTOR;


typedef struct _ATM_TRAFFIC_DESCRIPTOR_IE
{
	ATM_TRAFFIC_DESCRIPTOR		ForwardTD;
	ATM_TRAFFIC_DESCRIPTOR		BackwardTD;
	BOOLEAN						BestEffort;
} ATM_TRAFFIC_DESCRIPTOR_IE, *PATM_TRAFFIC_DESCRIPTOR_IE;


//
// values used for the BearerClass field in the Broadband Bearer Capability IE
//


#define BCOB_A					0x00	// Bearer class A
#define BCOB_C					0x01	// Bearer class C
#define BCOB_X					0x02	// Bearer class X

//
// values used for the TrafficType field in the Broadband Bearer Capability IE
//
#define TT_NOIND				0x00	// No indication of traffic type
#define TT_CBR					0x04	// Constant bit rate
#define TT_VBR					0x08	// Variable bit rate

//
// values used for the TimingRequirements field in the Broadband Bearer Capability IE
//
#define TR_NOIND				0x00	// No timing requirement indication
#define TR_END_TO_END			0x01	// End-to-end timing required
#define TR_NO_END_TO_END		0x02	// End-to-end timing not required

//
// values used for the ClippingSusceptability field in the Broadband Bearer Capability IE
//
#define CLIP_NOT				0x00	// Not susceptible to clipping
#define CLIP_SUS				0x20	// Susceptible to clipping

//
// values used for the UserPlaneConnectionConfig field in
// the Broadband Bearer Capability IE
//
#define UP_P2P					0x00	// Point-to-point connection
#define UP_P2MP					0x01	// Point-to-multipoint connection

//
// Broadband Bearer Capability
//
typedef struct _ATM_BROADBAND_BEARER_CAPABILITY_IE
{
	unsigned char			BearerClass;
	unsigned char			TrafficType;
	unsigned char			TimingRequirements;
	unsigned char			ClippingSusceptability;
	unsigned char			UserPlaneConnectionConfig;
} ATM_BROADBAND_BEARER_CAPABILITY_IE, *PATM_BROADBAND_BEARER_CAPABILITY_IE;

//
// values used for the HighLayerInfoType field in ATM_BHLI
//
#define BHLI_ISO				0x00	// ISO
#define BHLI_UserSpecific		0x01	// User Specific
#define BHLI_HighLayerProfile	0x02	// High layer profile (only in UNI3.0)
#define BHLI_VendorSpecificAppId 0x03	// Vendor-Specific Application ID

//
// Broadband High layer Information
//
typedef struct _ATM_BHLI_IE
{
	UINT32			HighLayerInfoType;		// High Layer Information Type
	UINT32			HighLayerInfoLength;	// number of bytes in HighLayerInfo
	unsigned char			HighLayerInfo[8];		// The value dependent on the
											// HighLayerInfoType field
} ATM_BHLI_IE, *PATM_BHLI_IE;

//
// values used for Layer2Protocol in B-LLI
//
#define BLLI_L2_ISO_1745		0x01	// Basic mode ISO 1745
#define BLLI_L2_Q921			0x02	// CCITT Rec. Q.921
#define BLLI_L2_X25L			0x06	// CCITT Rec. X.25, link layer
#define BLLI_L2_X25M			0x07	// CCITT Rec. X.25, multilink
#define BLLI_L2_ELAPB			0x08	// Extended LAPB; for half duplex operation
#define BLLI_L2_HDLC_ARM		0x09	// HDLC ARM (ISO 4335)
#define BLLI_L2_HDLC_NRM		0x0A	// HDLC NRM (ISO 4335)
#define BLLI_L2_HDLC_ABM		0x0B	// HDLC ABM (ISO 4335)
#define BLLI_L2_LLC				0x0C	// LAN logical link control (ISO 8802/2)
#define BLLI_L2_X75				0x0D	// CCITT Rec. X.75, single link procedure
#define BLLI_L2_Q922			0x0E	// CCITT Rec. Q.922
#define BLLI_L2_USER_SPECIFIED	0x10	// User Specified
#define BLLI_L2_ISO_7776		0x11	// ISO 7776 DTE-DTE operation

//
// values used for Layer3Protocol in B-LLI
//
#define BLLI_L3_X25				0x06	// CCITT Rec. X.25, packet layer
#define BLLI_L3_ISO_8208		0x07	// ISO/IEC 8208 (X.25 packet layer for DTE
#define BLLI_L3_X223			0x08	// X.223/ISO 8878
#define BLLI_L3_SIO_8473		0x09	// ISO/IEC 8473 (OSI connectionless)
#define BLLI_L3_T70				0x0A	// CCITT Rec. T.70 min. network layer
#define BLLI_L3_ISO_TR9577		0x0B	// ISO/IEC TR 9577 Network Layer Protocol ID
#define BLLI_L3_USER_SPECIFIED	0x10	// User Specified

//
// values used for Layer3IPI in struct B-LLI
//
#define BLLI_L3_IPI_SNAP		0x80	// IEEE 802.1 SNAP identifier
#define BLLI_L3_IPI_IP			0xCC	// Internet Protocol (IP) identifier

//
// Broadband Lower Layer Information
//
typedef struct _ATM_BLLI_IE
{
	UINT32						Layer2Protocol;
	unsigned char						Layer2Mode;
	unsigned char						Layer2WindowSize;
	UINT32						Layer2UserSpecifiedProtocol;
	UINT32						Layer3Protocol;
	unsigned char						Layer3Mode;
	unsigned char						Layer3DefaultPacketSize;
	unsigned char						Layer3PacketWindowSize;
	UINT32						Layer3UserSpecifiedProtocol;
	UINT32						Layer3IPI;
	unsigned char						SnapId[5];
} ATM_BLLI_IE, *PATM_BLLI_IE;


//
// Raw Information Element - the user can fill in whatever he wants
//
typedef struct _ATM_RAW_IE
{
	UINT32						RawIELength;
	UINT32						RawIEType;
	unsigned char						RawIEValue[1];
} ATM_RAW_IE, *PATM_RAW_IE;



typedef struct _ATM_VPIVCI
{
	UINT32						Vpi;
	UINT32						Vci;
} ATM_VPIVCI, *PATM_VPIVCI;

//
// ATM Service Category
//
#define	ATM_SERVICE_CATEGORY_CBR	1	// Constant Bit Rate
#define	ATM_SERVICE_CATEGORY_VBR	2	// Variable Bit Rate
#define	ATM_SERVICE_CATEGORY_UBR	4	// Unspecified Bit Rate
#define	ATM_SERVICE_CATEGORY_ABR	8	// Available Bit Rate

typedef UINT32	ATM_SERVICE_CATEGORY, *PATM_SERVICE_CATEGORY;


/*
 *  ATM specific Ioctl codes
 */
#define SIO_GET_NUMBER_OF_ATM_DEVICES   0x50160001
#define SIO_GET_ATM_ADDRESS             0xd0160002
#define SIO_ASSOCIATE_PVC               0x90160003
#define SIO_GET_ATM_CONNECTION_ID       0x50160004


//
// ATM flow parameters for use in specifying Media parameters
//
typedef struct _ATM_FLOW_PARAMETERS
{
	ATM_SERVICE_CATEGORY		ServiceCategory;
	UINT32						AverageCellRate;			// in cells/sec
	UINT32						PeakCellRate;				// in cells/sec
	UINT32						MinimumCellRate;			// in cells/sec (ABR MCR)
	UINT32						InitialCellRate;			// in cells/sec (ABR ICR)
	UINT32						BurstLengthCells;			// in cells
	UINT32						MaxSduSize;					// MTU in bytes
	UINT32						TransientBufferExposure;	// in cells (ABR TBE)
	UINT32						CumulativeRMFixedRTT;		// in microseconds (ABR FRTT)
	unsigned char						RateIncreaseFactor;			// UNI 4.0 coding (ABR RIF)
	unsigned char						RateDecreaseFactor;			// UNI 4.0 coding (ABR RDF)
	UINT16						ACRDecreaseTimeFactor;		// UNI 4.0 coding (ABR ADTF)
	unsigned char						MaximumCellsPerForwardRMCell; // UNI 4.0 coding (ABR Nrm)
	unsigned char						MaximumForwardRMCellInterval; // UNI 4.0 coding (ABR Trm)
	unsigned char						CutoffDecreaseFactor;		// UNI 4.0 coding (ABR CDF)
	unsigned char						Reserved1;					// padding
	UINT32						MissingRMCellCount;			// (ABR CRM)
	UINT32						Reserved2;
	UINT32						Reserved3;
} ATM_FLOW_PARAMETERS, *PATM_FLOW_PARAMETERS;


// 
#define ENABLE_TRANSMIT				0x00000001
#define ENABLE_RECEIVE				0x00000002


//
// ATM Specific Media parameters - this is the Media specific structure for ATM
// that goes into the ACtivate VC request.
//
typedef struct _ATM_PARAMETERS
{
	UINT32                      VcHandle;
UINT32                      Flags;
	ATM_VPIVCI					ConnectionId;
	ATM_AAL_TYPE				AALType;
	ATM_FLOW_PARAMETERS			Transmit;
} ATM_PARAMETERS, *PATM_PARAMETERS;


//  Bit 0 in Reserved1 in ATM_FLOW_PARAMETERS is reserved.
#define ATM_FLOW_PARAMS_RSVD1_MPP	0x01

#ifndef SAP_FIELD_ABSENT
#define SAP_FIELD_ABSENT		((UINT32)0xfffffffe)
#endif

#ifndef SAP_FIELD_ANY
#define SAP_FIELD_ANY			((UINT32)0xffffffff)
#endif



//
// the structure for returning the supported VC rates from the miniport,
// returned in response to OID_ATM_SUPPORTED_VC_RATES
//
typedef struct _ATM_VC_RATES_SUPPORTED
{
	UINT32						MinCellRate;
	UINT32						MaxCellRate;
} ATM_VC_RATES_SUPPORTED, *PATM_VC_RATES_SUPPORTED;



//
//  Physical Line Speeds in bits/sec.
//
#define ATM_PHYS_RATE_SONET_STS3C						155520000
#define ATM_PHYS_RATE_IBM_25						 	 25600000

//
//  ATM cell layer transfer capacities in bits/sec. This is the throughput
//  available for ATM cells, after allowing for physical framing overhead.
//
#define ATM_CELL_TRANSFER_CAPACITY_SONET_STS3C			149760000
#define ATM_CELL_TRANSFER_CAPACITY_IBM_25			 	 25125926



//
//  User data rate in units of 100 bits/sec. This is returned in response to
//  the OID_GEN_LINK_SPEED query. This is the effective rate of
//  transfer of data available to the ATM layer user, after allowing for
//  the ATM cell header.
//
#define ATM_USER_DATA_RATE_SONET_155					1356317
#define ATM_USER_DATA_RATE_IBM_25			            227556

// 
#define	DFLT_155_MAXBANDWIDTH							353208		// cells per second at 155mhz


#endif	//	_ATM_H_


